{% load static %}
<link rel="stylesheet" href="{% static 'django_spellbook/root.css' %}">
<link rel="stylesheet" href="{% static 'django_spellbook/utilities.css' %}">
<link rel="stylesheet" href="{% static 'django_spellbook/styles.css' %}">
